#include <ESP8266WiFi.h>
#include <ESP8266WebServer.h>
#include <EEPROM.h>
#include "Index.h"
#include "WEBset.h"
extern "C" {
#include "user_interface.h"
}

#define DNS
#define Taster 2
#define Relay 0

IPAddress apIP(192, 168, 168, 30);

#if defined DNS
#include <DNSServer.h>
const byte DNS_PORT = 53;
DNSServer dnsServer;
#endif

char ssid[32] = "fhgdhfghd\0";
char passwort[64] = "gghdfgh\0";

char BenuzName[32] = "admin\0";  // Name beim Autetifizierung
char BenuzPasswort[32] = "gghdfgh\0"; // Password wird imm Setup festgelegt
ESP8266WebServer server;
EspClass esp;
int z = 0;                   //Aktuelle EEPROM-Adresse zum lesen
#include "Setup.h"

int i = 0;
int val = 1;
String Temp = "";
boolean AP = 0; // Acsespoint Modus aus

void setup()                // Wird 1 Mal beim Start ausgefuehrt
{
  char inser;               // Serielle daten ablegen
  String nachricht = "";    //  Setup Formular
  pinMode(Relay, OUTPUT);
  pinMode(Taster, INPUT_PULLUP);
  digitalWrite(Relay, 1);

  EEPROM.begin(250);                                 // EEPROM initialisieren mit 200 Byts
  Serial.begin(115200);

  while (Serial.available())inser = Serial.read();  // Setup Weiche, Erkennen ob TXD/RXD gebrückt sind
  Serial.println("Start");
  delay(10);
  inser = Serial.read();
  if (inser != 'S') goto weiter;
  inser = Serial.read();
  if (inser != 't') goto weiter;
  Einstellen();

weiter:
  z = 0;
  LeseEeprom(ssid, sizeof(ssid));        // EEPROM lesen
  if (ssid[0] == 255) // Wenn EEPROM leer ist dann:
  {
    for (i = 0; i < 6; i++) EEPROM.write(i, '\0');
    EEPROM.commit();
    ssid[0] = '\0';
  }
  LeseEeprom(passwort, sizeof(passwort));
  LeseEeprom(BenuzPasswort, sizeof(BenuzPasswort));


  if (ssid[0] == '\0')
  {
    Serial.println("Starte in Access Point modus" );
    Serial.println("IP http://192.168.168.30");
    Serial.print("SSID: WebSchalter, Passwort: ");
    Serial.println(passwort);
    WiFi.mode(WIFI_AP);      // access point modus
    WiFi.softAPConfig(apIP, apIP, IPAddress(255, 255, 255, 0));
    strncpy(ssid, "WebSchalter\0", 13);
    WiFi.softAP(&ssid[0], &passwort[0]);  // Name des Wi-Fi netzes
#if defined DNS
    dnsServer.start(DNS_PORT, "*", apIP);
#endif
    AP = 1;
  }
  else
  {
    Serial.print("Verbinde mit ");
    Serial.println(ssid);
    WiFi.mode(WIFI_STA);
    WiFi.begin(ssid, passwort);

    while (WiFi.status() != WL_CONNECTED)
    {
      delay(500);
      Serial.print("O");
    }

    Serial.begin(115200);
    Serial.println("");
    Serial.println("Mit Wlan verbunden");
    Serial.println("IP Adresse: ");
    Serial.println(WiFi.localIP());
  }// Wenn nicht SSID 0

  if (!BenuzPasswort[0])Serial.println("Serverauthentifizierung ausgeschaltet");
  else
  {
    Serial.println("Serverauthentifizierung eingeschaltet");
    Serial.print("Benutzername : ");
    Serial.println(BenuzName);
    Serial.print("Benutzerpasswort : ");
    Serial.println(BenuzPasswort);
  }

  //  Bechandlung der Ereignissen anschlissen
  server.on("/", Ereignis_Index);
  server.on("/1.html", Ereignis_SchalteON);
  server.on("/0.html", Ereignis_SchalteOFF);
  server.on("/frame", Ereignis_Frame);
  server.onNotFound ( handleNotFound );

  server.begin();  // Starte den Server
  Serial.print("HTTP Server gestartet");
}

void Ereignis_SchalteON()    // Wird ausgefuehrt wenn "http://<ip address>/schalt" aufgerufen wurde
{
  if (BenuzPasswort[0]) // Wenn Benutzerpasswort angegeben wurde
  {
    if (!server.authenticate(&BenuzName[0], &BenuzPasswort[0]))
      return server.requestAuthentication();
  }
  val = 0;                   // Relais Aus
  digitalWrite(Relay, val);      // GPIO0
  Temp = html1 + String((val) ? "/1.html" : "/0.html"); // Passende link bilden und in nechster Zeile Farbe
  Temp += html2 + String((val) ? "BACKGROUND-COLOR: DarkGray;" : "BACKGROUND-COLOR: Chartreuse;") + html3;
  server.sendHeader("Cache-Control", "no-cache");
  server.send(200, "text/html", Temp);
}

void Ereignis_SchalteOFF()    // Wird ausgefuehrt wenn "http://<ip address>/schalt" aufgerufen wurde
{
  if (BenuzPasswort[0]) // Wenn Benutzerpasswort angegeben wurde
  {
    if (!server.authenticate(&BenuzName[0], &BenuzPasswort[0]))
      return server.requestAuthentication();
  }
  val = 1;                     // Relais Ein
  digitalWrite(Relay, val);        // GPIO0
  Temp = html1 + String((val) ? "/1.html" : "/0.html");
  Temp += html2 + String((val) ? "BACKGROUND-COLOR: DarkGray;" : "BACKGROUND-COLOR: Chartreuse;") + html3;
  server.sendHeader("Cache-Control", "no-cache");
  server.send(200, "text/html", Temp);
}

void Ereignis_Index()    // Wird ausgeuehrt wenn "http://<ip address>/" aufgerufen wurde
{
  if (BenuzPasswort[0]) // Wenn Benutzerpasswort angegeben wurde
  {
    if (!server.authenticate(&BenuzName[0], &BenuzPasswort[0]))
      return server.requestAuthentication();
  }
  Temp = html1 + String((val) ? "/1.html" : "/0.html");
  Temp += html2 + String((val) ? "BACKGROUND-COLOR: DarkGray;" : "BACKGROUND-COLOR: Chartreuse;") + html3;
  server.sendHeader("Cache-Control", "no-cache");
  server.send(200, "text/html", Temp);
}

void Ereignis_Frame()    // Wird ausgefuehrt wenn "http://<ip address>/" aufgerufen wurde
{
  if (BenuzPasswort[0]) // Wenn Benutzerpasswort angegeben wurde
  {
    if (!server.authenticate(&BenuzName[0], &BenuzPasswort[0]))
      return server.requestAuthentication();
  }
  server.sendHeader("Cache-Control", "no-cache");
  server.send(200, "text/html", htmlFrame );
}


void loop()
{
#if defined DNS
  if (AP)dnsServer.processNextRequest();
#endif
  if (!digitalRead(Taster))            // Taster betätigt
  {
    delay(50);                         // Nach 100mS immer noch betätigt
    if (!digitalRead(Taster))
    {
      val = !val;
      digitalWrite(Relay, val);
      while (!digitalRead(Taster))server.handleClient();  // Warten bis der Taster losgelassen wird
    }
  }
  server.handleClient();                           // Server Ereignisse abarbeiten
}




